package meerkat.util;

import com.google.protobuf.ByteString;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.*;
import com.google.protobuf.Timestamp;

import java.math.BigInteger;
import java.security.SignatureException;
import java.util.Arrays;
import java.util.Random;

/**
 * Created by Arbel Deutsch Peled on 21-Feb-16.
 * This class contains methods used to generate random Bulletin Board Messages
 */
public class BulletinBoardMessageGenerator {

    private Random random;

    public BulletinBoardMessageGenerator(Random random) {
        this.random = random;
    }

    private byte randomByte(){
        return (byte) random.nextInt();
    }

    private String randomString(){
        return new BigInteger(130, random).toString(32);
    }

    /**
     * Generates a complete instance of a BulletinBoardMessage
     * @param signers contains the (possibly multiple) credentials required to sign the message
     * @param timestamp contains the time used in the message
     * @param dataSize is the length of the data contained in the message
     * @param tagNumber is the number of tags to generate
     * @return a random, signed Bulletin Board Message containing random data and tags and the given timestamp
     */

    public BulletinBoardMessage generateRandomMessage(DigitalSignature[] signers, Timestamp timestamp, int dataSize, int tagNumber)
            throws SignatureException {

        // Generate random data.

        byte[] data = new byte[dataSize];
        String[] tags = new String[tagNumber];

        for (int i = 0; i < dataSize; i++) {
            data[i] = randomByte();
        }

        for (int i = 0; i < tagNumber; i++) {
            tags[i] = randomString();
        }

        UnsignedBulletinBoardMessage unsignedMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .setData(ByteString.copyFrom(data))
                        .setTimestamp(timestamp)
                        .addAllTag(Arrays.asList(tags))
                        .build();

        BulletinBoardMessage.Builder messageBuilder =
                BulletinBoardMessage.newBuilder()
                    .setMsg(unsignedMessage);

        for (int i = 0 ; i < signers.length ; i++) {
            signers[i].updateContent(unsignedMessage);
            messageBuilder.addSig(signers[i].sign());
        }

        return messageBuilder.build();

    }

    /**
     * Generates a complete instance of a BulletinBoardMessage
     * @param signers contains the (possibly multiple) credentials required to sign the message
     * @param dataSize is the length of the data contained in the message
     * @param tagNumber is the number of tags to generate
     * @return a random, signed Bulletin Board Message containing random data, tags and timestamp
     */

    public BulletinBoardMessage generateRandomMessage(DigitalSignature[] signers, int dataSize, int tagNumber)
            throws SignatureException {

        Timestamp timestamp = Timestamp.newBuilder()
                .setSeconds(random.nextLong())
                .setNanos(random.nextInt())
                .build();

        return generateRandomMessage(signers, timestamp, dataSize, tagNumber);

    }

}
