package meerkat;

import com.google.common.util.concurrent.FutureCallback;
import meerkat.bulletinboard.AsyncBulletinBoardClient;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.VoterRegistry.VoterID;
import meerkat.protobuf.VoterRegistry.VoterInfo;
import meerkat.protobuf.VoterRegistry.VoterRegistryMessage;

import java.io.IOException;
import java.security.SignatureException;
import java.util.Collection;
import java.util.List;

/**
 * Created by Vladimir Eliezer Tokarev on 1/22/2016.
 * provides voters management options
 */
public interface VoterRegistry {

    /**
     * Initialize the voter registry
     * @param signers collection of singers every object will sign every output message
     * @param communicator  the object which communicates with the BulletinBoardServer
     */
    void init(Collection<DigitalSignature> signers, AsyncBulletinBoardClient communicator);

    /**
     * Adds new voter to the bulletin-board
     *      Passes true to callBack.handleResult  if the actions succeeded else
     *      call the onFailure method of the callback with the arisen error
     *
     * @param voterInfo    protobuff object that represents voter information
     * @param callBack     when the adding voter done callBack.handleResult will be called
     * @throws SignatureException
     */
    void addVoter(VoterInfo voterInfo, FutureCallback<Boolean> callBack) throws SignatureException;

    /**
     * Adding given voter to given group
     *        Passes the group  to callBack.handleResult  if the actions succeeded else
     *        call the onFailure method of the callback with the arisen error
     *
     * @param voterRegistryMessage protobuff object that is coupling of voterId to groupId
     * @param callBack when the adding voter done callBack.handleResult will be called
     * @throws SignatureException
     * @throws IOException
     */
    void setVoterGroups(VoterRegistryMessage voterRegistryMessage, FutureCallback<Boolean>  callBack) throws SignatureException, IOException;

    /**
     * Sets that the voter have voted
     *          Passes true to callBack.handleResult  if the actions succeeded else
     *          call the onFailure method of the callback with the arisen error
     *
     * @param voterId  protobuff object that represent the voter that have voted
     * @param callBack when the adding voter done callBack.handleResult will be called
     * @throws SignatureException
     */
    void setVoted(VoterID voterId, FutureCallback<Boolean>  callBack) throws SignatureException;

    /**
     * Requests all the groups that the given id voter is in
     *         Passes wanted groups to callback.handleResult if the actions succeeded else
     *         call the onFailure method of the callback with the arisen error
     *
     * @param voterID protobuff object that represent the voter
     * @param callBack when the adding voter done callBack.handleResult will be called
     */
    void getGroups(VoterID voterID, FutureCallback<List<meerkat.protobuf.VoterRegistry.GroupID>>  callBack);

    /**
     * Retrieves VoterInfo protobuff that represents voter
     *          Passes wanted data to callback.handleResult if the actions succeeded else
     *          call the onFailure method of the callback with the arisen error
     *
     * @param voterID  protobuff object that represent the voter
     * @param callBack when the adding voter done callBack.handleResult will be called
     */
    void getVoter(VoterID voterID, FutureCallback<VoterInfo>  callBack);

    /**
     * Checks if the given voter (by his id) have already voted
     *        passes true of the voter have been voted or not else
     *        call the onFailure method of the callback with the arisen error
     *
     * @param voterID   protobuff object that represent the voter
     * @param callBack  method that will be called with the when the result will be found
     */
    void hasVoted(VoterID voterID, FutureCallback<Boolean>  callBack);
}

